/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtContacts module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QCONTACTFETCHHINT_P_H
#define QCONTACTFETCHHINT_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/qshareddata.h>
#include <QtCore/qsize.h>
#include <QtCore/qstringlist.h>

#include <QtContacts/qcontactdetail.h>
#include <QtContacts/qcontactfetchhint.h>

QT_BEGIN_NAMESPACE_CONTACTS

class QContactFetchHintPrivate : public QSharedData
{
public:
    QContactFetchHintPrivate()
        : QSharedData(),
        m_optimizationHints(QContactFetchHint::AllRequired),
        m_maxCount(-1)
    {
    }

    QContactFetchHintPrivate(const QContactFetchHintPrivate& other)
        : QSharedData(other),
        m_typesHint(other.m_typesHint),
        m_relationshipsHint(other.m_relationshipsHint),
        m_optimizationHints(other.m_optimizationHints),
        m_maxCount(other.m_maxCount)
    {
    }

    ~QContactFetchHintPrivate()
    {
    }

    QList<QContactDetail::DetailType> m_typesHint;
    QStringList m_relationshipsHint;
    QSize m_preferredImageSize;
    QContactFetchHint::OptimizationHints m_optimizationHints;
    int m_maxCount;
};

QT_END_NAMESPACE_CONTACTS

#endif // QCONTACTFETCHHINT_P_H
