/* This file was generated by syncqt. */
#ifndef QT_QTCONTACTS_VERSION_H
#define QT_QTCONTACTS_VERSION_H

#define QTCONTACTS_VERSION_STR "5.0.0"

#define QTCONTACTS_VERSION 0x050000

#endif // QT_QTCONTACTS_VERSION_H
