/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtContacts module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QTCONTACTS_H
#define QTCONTACTS_H

// this file includes all of the public header files
// provided by the Qt Contacts API

#include <QtContacts/qcontactsglobal.h>                    // global exports
#include <QtContacts/qcontactmanager.h>                    // manager
#include <QtContacts/qcontactmanagerengine.h>              // manager backend
#include <QtContacts/qcontactmanagerenginefactory.h>       // manage backend instantiator
#include <QtContacts/qcontact.h>                           // contact
#include <QtContacts/qcontactid.h>                         // contact identifier
#include <QtContacts/qcontactobserver.h>                   // contact observer
#include <QtContacts/qcontactdetail.h>                     // contact detail
#include <QtContacts/qcontactdetails.h>                    // leaf detail classes
#include <QtContacts/qcontactfetchhint.h>                  // backend optimization hint class
#include <QtContacts/qcontactfilter.h>                     // contact filter
#include <QtContacts/qcontactfilters.h>                    // leaf filter classes
#include <QtContacts/qcontactsortorder.h>                  // contact sorting
#include <QtContacts/qcontactaction.h>                     // actions
#include <QtContacts/qcontactactiondescriptor.h>           // action descriptors
#include <QtContacts/qcontactactionfactory.h>              // action factory
#include <QtContacts/qcontactactiontarget.h>               // action target (contact + detail(s))
#include <QtContacts/qcontactabstractrequest.h>            // asynchronous request
#include <QtContacts/qcontactrequests.h>                   // request leaf classes
#include <QtContacts/qcontactrelationship.h>               // contact relationships

QT_BEGIN_NAMESPACE_CONTACTS
QT_END_NAMESPACE_CONTACTS

#endif // QTCONTACTS_H
