/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtContacts module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QCONTACT_P_H
#define QCONTACT_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/qlist.h>
#include <QtCore/qmap.h>
#include <QtCore/qshareddata.h>

#include <QtContacts/qcontact.h>
#include <QtContacts/qcontactdetail.h>
#include <QtContacts/qcontactid.h>
#include <QtContacts/qcontactrelationship.h>
#include <QtContacts/qcontactcollectionid.h>

QT_BEGIN_NAMESPACE_CONTACTS

class QContactData : public QSharedData
{
public:
    QContactData()
        : QSharedData()
    {
    }

    QContactData(const QContactData& other)
        : QSharedData(other),
        m_id(other.m_id),
        m_collectionId(other.m_collectionId),
        m_details(other.m_details),
        m_relationshipsCache(other.m_relationshipsCache),
        m_preferences(other.m_preferences)
    {
    }

    ~QContactData() {}

    QContactId m_id;
    QContactCollectionId m_collectionId;
    QList<QContactDetail> m_details;
    QList<QContactRelationship> m_relationshipsCache;
    QMap<QString, int> m_preferences;

    // Helper function
    void removeOnly(QContactDetail::DetailType type);
    void removeOnly(const QSet<QContactDetail::DetailType>& types);

    // Trampoline
    static QSharedDataPointer<QContactData>& contactData(QContact& contact) {return contact.d;}
};

QT_END_NAMESPACE_CONTACTS

#endif // QCONTACT_P_H
